@@ -6,7 +6,7 @@ Mojo::IRC - IRC Client for the Mojo IOLoop
 
 =head1 VERSION
 
-0.15
+0.16
 
 =head1 SYNOPSIS
 
@@ -128,7 +128,7 @@ use Mojo::Base 'Mojo::EventEmitter';
 use Mojo::IOLoop;
 use File::Basename 'dirname';
 use File::Spec::Functions 'catfile';
-use IRC::Utils;
+use IRC::Utils   ();
 use Parse::IRC   ();
 use Scalar::Util ();
 use Unicode::UTF8;
@@ -137,7 +137,7 @@ use constant DEFAULT_CERT => $ENV{MOJO_IRC_CERT_FILE} || catfile dirname(__FILE_
 use constant DEFAULT_KEY  => $ENV{MOJO_IRC_KEY_FILE}  || catfile dirname(__FILE__), 'mojo-irc-client.key';
 use constant OFFLINE => $ENV{MOJO_IRC_OFFLINE} ? 1 : 0;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 my %CTCP_QUOTE = ("\012" => 'n', "\015" => 'r', "\0" => '0', "\cP" => "\cP");
 
@@ -290,9 +290,9 @@ sub connect {
         close => sub {
           $self or return;
           warn "[$self->{debug_key}] : close\n" if DEBUG;
-          $self->emit('close');
           delete $self->{stream};
           delete $self->{stream_id};
+          $self->emit('close');
         }
       );
       $stream->on(
@@ -366,12 +366,12 @@ sub disconnect {
       "QUIT\r\n",
       sub {
         $self->{stream}->close;
-        $self->$cb;
+        $self->$cb if $cb;
       }
     );
   }
   else {
-    $self->$cb;
+    $self->$cb if $cb;
   }
 
   $self;
@@ -441,14 +441,11 @@ Will respond to the sender with the difference in time.
 
 sub ctcp_ping {
   my ($self, $message) = @_;
-  my $t0 = $message->{params}[1] || '';
+  my $ts   = $message->{params}[1];
+  my $nick = IRC::Utils::parse_user($message->{prefix});
 
-  return $self unless $t0 =~ /^\d+$/;
-  return $self->write(
-    'NOTICE',
-    $message->{params}[0],
-    $self->ctcp(sprintf "Ping reply from %s: %s second(s)", $self->nick, time - $t0),
-  );
+  return $self unless $ts;
+  return $self->write('NOTICE', $nick, $self->ctcp(PING => $ts));
 }
 
 =head2 ctcp_time
@@ -463,8 +460,9 @@ NOTE! The localtime format may change.
 
 sub ctcp_time {
   my ($self, $message) = @_;
+  my $nick = IRC::Utils::parse_user($message->{prefix});
 
-  $self->write(NOTICE => $message->{params}[0], $self->ctcp(TIME => scalar localtime));
+  $self->write(NOTICE => $nick, $self->ctcp(TIME => scalar localtime));
 }
 
 =head2 ctcp_version
@@ -479,8 +477,9 @@ NOTE! Additional information may be added later on.
 
 sub ctcp_version {
   my ($self, $message) = @_;
+  my $nick = IRC::Utils::parse_user($message->{prefix});
 
-  $self->write(NOTICE => $message->{params}[0], $self->ctcp(VERSION => 'Mojo-IRC', $VERSION));
+  $self->write(NOTICE => $nick, $self->ctcp(VERSION => 'Mojo-IRC', $VERSION));
 }
 
 =head2 irc_nick
